import javax.swing.*;
import java.awt.*;
import java.awt.geom.*;
import java.util.*;

/** Analoganzeige fuer das Uhrenprogramm
  **/
public class AnalogAnzeige_1 extends JPanel implements Anzeige {
  
  /** Dieses Uhrwerk wird fuer die Zeitdarstellung verwendet. */
  private Uhrwerk uhrwerk;
  
  /** Die darzustellenden Stunden (0-11) */
  private int stunden;
  
  /** Die darzustellenden Minuten (0-59) */
  private int minuten;
  
  /** Die darzustellenden Sekunden (0-59) */
  private int sekunden;
  
  /** Ist es nachmittags (PM) ? */
  private boolean zeigePM;
  
  /** Konstruktor */
  public AnalogAnzeige_1() {
    // Veraendere das Aussehen 
    setBackground(Color.BLACK);
    setForeground(Color.GREEN);
    // Sorge dafuer, dass die Komponente
    // eine gewisse bevorzugte Groesse hat
    setPreferredSize(new Dimension(200,200));
  }
  
  /** Nennt der Anzeige das Uhrwerk, das verwendet werden
    * soll.
    * @param uhr das zu verwendende Uhrwerk
    **/
  public void setUhrwerk(Uhrwerk uhr) {
    uhrwerk = uhr;
  }
  
  /** Weist die Anzeige an, die aktuelle Uhrzeit aus dem
    * Uhrwerk auszulesen und darzustellen.
    **/
  public void zeigeZeit() {
    // Extrahiere Stunden, Minuten und Sekunden
    Date date = uhrwerk.getZeit();
    Calendar calendar = new GregorianCalendar();
    calendar.setTime(date);
    stunden = calendar.get(Calendar.HOUR);
    minuten = calendar.get(Calendar.MINUTE);
    sekunden = calendar.get(Calendar.SECOND);
    zeigePM = calendar.get(Calendar.AM_PM) == Calendar.PM;
    // Dann rufe die repaint-Methode auf
    repaint();
  }  
  
  /** Malt die Uhr auf dem Bildschirm */
  public void paint(Graphics graphics) {
    // Zeichne nicht direkt auf dem Bildschirm, sondern
    // verwende ein so genanntes Offscreen-Image
    int width = getWidth();
    int height = getHeight();
    Image bild = createImage(width,height);
    Graphics g = bild.getGraphics();
    // Berechne den Mittelpunkt unseres Uhrenkreises
    int mitteX = width/2;
    int mitteY = height/2;
    int radius = Math.min(width,height) / 2 - 10;
    // Zuerst einmal malen wir den Hintergrund
    g.setColor(getBackground());
    g.fillRect(0,0,width,height);    
    // Nun zum Zifferblatt
    g.setColor(getForeground());
    g.drawOval(mitteX - radius, mitteY - radius,
      2 * radius, 2 * radius);
    // Im Falle von "PM", zeige einen kleinen Punkt an
    if (zeigePM)
      g.fillOval(width - 20, height - 20 , 10 , 10);
    // Zu guter Letzt zeichnen wir das Offscreen-Image
    // in der wirklichen Komponente
    graphics.drawImage(bild,0,0,this);
  }
  
}
