/** Diese Klasse synchronisiert zwei Uhren
  * in einem vorgegebenen Zeittakt.
  **/
public class Synchronisierer implements Steuerung, Runnable {
  /** Die Uhr, von der die Zeit gelesen wird */
  private Uhrwerk quelle;
  /** Die Uhr, die gestellt werden soll */
  private Uhrwerk ziel;
  /** Der Takt, in dem die Aktualisierung vonstatten gehen
    * soll (in Millisekunden, mindestens 100)
    **/
  private int takt;
  /** Der Thread, der gerade aktiv ist */
  private Thread thread;
  /** Konstruktor. */
  public Synchronisierer(Uhrwerk ziel) {
    this.ziel = ziel;
    takt = 100;
  }
  /** Setze den fuer die Aktualisierung zu verwendenden Takt 
    * (in Millisekunden, mindestens 100)
    **/
  public void setTakt(int takt) {
    boolean aktiv = thread != null;
    if (aktiv) 
      beenden();
    this.takt = Math.max(100,takt);
    if (aktiv)
      aktivieren();
  }
  /** Setze die fuer die Aktualisierung verwendete Quelle */
  public void setQuelle(Uhrwerk quelle) {
    boolean aktiv = thread != null;
    if (aktiv) 
      beenden();
    this.quelle = quelle;
    if (aktiv)
      aktivieren();
  }
  /** Gib das fuer die Aktualisierung verwendete Ziel zurueck. */
  public Uhrwerk getZiel() {
    return ziel;
  }
  /** Diese Methode signalisiert der Steuerung, dass
    * sie mit ihrer Arbeit beginnen soll.
    **/
  public void aktivieren() {
    synchronized(this) {
      if (thread != null) 
        return;
      thread = new Thread(this);
      thread.start();
    }
  }
  /** Diese Methode signalisiert der Steuerung, dass
    * sie ihre Arbeit jetzt beenden kann.
    **/
  public void beenden() {
    synchronized(this) {
      if (thread == null) 
        return;
      thread.interrupt();
      thread = null;
    }
  }
  /** Diese Methode fuehrt die eigentliche Aktualisierung 
    * zyklisch durch.
    **/
  public void run() {
    // Kopiere die fuer diesen Thread gueltigen
    // Quelle und Takt in lokale Felder
    Uhrwerk quelle = null;
    int takt = 0;
    synchronized(this) {
      quelle = this.quelle;
      takt = this.takt;
    }
    // Wir durchlaufen eine (beinahe) Endlos-Schleife
    while(!Thread.currentThread().isInterrupted() && quelle != null) {
      // Aktualisiere die Zeit 
      synchronized(this) {
        ziel.setZeit(quelle.getZeit());
      }
      // Schlafe fuer ein weilchen
      // (nicht synchronisiert!)
      try {
        Thread.sleep(takt);
      }
      catch(InterruptedException e) {
        Thread.currentThread().interrupt();
      }
    }
  }  
}
