import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;

/** Diese Klasse repraesentiert eine Stange, auf
  * der einzelne Scheiben aufgereiht werden koennen.
  **/
public class Stange {
  
  /** Die einzelnen Scheiben werden in einer Liste
    * abgelegt.
    **/
  private List<Scheibe> scheiben = new ArrayList<Scheibe>();
  
  /** Fuegt der Stange genau dann eine einzelne Scheibe  
    * hinzu, wenn nicht schon eine kleinere Scheibe auf 
    * der Stange sitzt.
    * @param scheibe die hinzugefuegte Scheibe
    * @return true, wenn das Einfuegen erfolgreich war
    **/
  public boolean fuegeEin(Scheibe scheibe) {
    // Test: haben wir versucht, null einzufuegen?
    if (scheibe == null)
      return false;
    // Test: ist die Stange leer?
    if (!scheiben.isEmpty()) {
      // Hole die oberste Scheibe aus der Liste
      Scheibe oben = (Scheibe) scheiben.get(0);
      // Vergleiche die Breiten der Scheiben
      if (oben.getBreite() < scheibe.getBreite()) 
        return false;
    }
    // Fuehre die Operation durch
    scheiben.add(0,scheibe);
    return true;
  }
  
  /** Entfernt das oberste Element vom Turm.
    * @return das oberste Element, null, falls
    * die Stange leer ist.
    **/
  public Scheibe entferne() {
    if (scheiben.isEmpty()) {
      return null;
    }
    return (Scheibe)scheiben.remove(0);
  }
  
  /** Gib die Hoehe des Turms aus 
    * @return die Anzahl der Scheiben, die auf der Stange
    * aufgereiht sind.
    **/
  public int getHoehe() {
    return scheiben.size();
  }  
  
  /** Gibt eine String-Repraesentation des Objektes zurueck.
    * @return eine textuelle Darstellung der Stange
    **/
  public String toString() {
    // Wir speichern das Ergebnis in einem StringBuffer zwischen
    StringBuffer res = new StringBuffer("||=");
    // ... und iterieren durch die Liste
    for (Iterator it = scheiben.iterator(); it.hasNext(); ) {
      res.insert(2,it.next());  // impliziter Aufruf von toString()
      res.insert(2,'=');
    }
    // Fertig :-)
    return res.toString();
  }  
}
