/** Dieses Programm loest das Tuerme-von-Hanoi-Problem
  * und gibt die Loesung auf dem Bildschirm aus. 
  **/
  
public class Hanoi {
  
  /** Ein Zaehler fuer die Zuege */
  private int zaehler;
  
  /** Die drei Tuerme */
  private Stange[] stangen = {
    new Stange(),new Stange(),new Stange()
  };
  
  /** Konstruktor 
    * @param anzahl die Anzahl der Scheiben
    **/
  public Hanoi(int anzahl) {
    for (int i = anzahl; i > 0; i--) 
      stangen[0].fuegeEin(new Scheibe(i));
  }
  
  /** Liefert eine String-Darstellung des
    * aktuellen Zustandes.
    * @return die drei Stangen in textueller Darstellung
    **/
  public String toString() {
    StringBuffer res = new StringBuffer();
    res.append("Start:       ");
    res.append(stangen[0].toString());
    res.append('\n');
    res.append("Ziel:        ");
    res.append(stangen[1].toString());
    res.append('\n');
    res.append("Hilfsstange: ");
    res.append(stangen[2].toString());
    return res.toString();
  }
 
  /** main-Methode: Erhaelt die Anzahl der Scheiben als
    * Kommandozeilenparameter und startet den Algorithmus.
    **/
  public static void main(String[] args) {
    if (args.length != 1) {
      System.out.println("Korrekter Aufruf: Hanoi <Anzahl Scheiben>");
      return;
    }
    int anzahl = Integer.parseInt(args[0]);
    new Hanoi(anzahl).spiele();
  }  

  /** Der eigentliche (rekursive) Verschiebe-Algorithmus.
    * @param kleinste die kleinste zu verschiebende Scheibe
    * @param groesste die groesste zu verschiebende Scheibe
    * @param start index der Stange, von der wir starten
    * @param ziel index der Stange, auf die wir schieben
    */    
  private void rekursion(int kleinste,int groesste,int start,int ziel){
    // Falls kleinste == groesste ist, sind wir am Ende der
    // Rekursion (und koennen direkt verschieben)
    if ( kleinste == groesste ) {
      // Die eigentliche Verschiebung
      boolean ok = stangen[ziel].fuegeEin(stangen[start].entferne());
      assert ok;
      // Gib den Zustand der Tuerme aus
      System.out.println("Zug : " + (++zaehler));
      System.out.println(this);
      System.out.println();
    }
    // Andernfalls muessen wir etwas mehr tun    
    else { 
      // Schiebe die kleineren Scheiben auf einen anderen Turm
      rekursion(kleinste, groesste - 1, start, 3 - start - ziel);
      // Dann verschiebe die groesste Scheibe
      rekursion(groesste,groesste,start,ziel);
      // Danach schiebe die ganzen kleineren ebenfalls aufs Ziel
      rekursion(kleinste, groesste - 1, 3 - start - ziel, ziel);
    } 
  }

  /** Fuehre den "Tuerme von Hanoi"-Algorithmus durch */
  public void spiele() {
    if (zaehler > 0)
      return;
    System.out.println("Ausgangszustand:");
    System.out.println(this);
    System.out.println();
    rekursion(1,stangen[0].getHoehe(),0,1);
  }
  
}
